import React, { Component } from "react"
import { Form, Layout, Spin } from "antd"
import UserForm from "./UserForm"
import { allUsers, allGenders } from "./constant"
import Container from "wp-container"
import RecordSelector from "wp-record-selector"

class User extends Component {
  columns = [
    {
      title: "Id",
      dataIndex: "id",
      key: "id",
      width: '20%',
      sorter: (a, b) => NullAsEmpty(a.id.toString()).localeCompare(NullAsEmpty(b.id.toString()))
    }, 
    {
      title: "Username",
      dataIndex: "username",
      key: "username",
      width: '30%',
      sorter: (a, b) => NullAsEmpty(a.username).localeCompare(NullAsEmpty(b.username))
      },
    {
      title: "Name",
      dataIndex: "name",
      key: "name",
      width: '50%',
      sorter: (a, b) => NullAsEmpty(a.name).localeCompare(NullAsEmpty(b.name))
    },
  ]

  state = {
    user: {},
    users: [],
    genders: [],
    spinMessage: null,
  }

  componentWillMount() {
    this.setState({
      users: allUsers,
      genders: allGenders
    })
  }

  handleSelectRecord = (record) => {
    const { users, user } = this.state
    const us = users.find(
      (u) => u.id === record.id
    )
    this.setState({
      user: us,
      undo: user,
      spinMessage: null,
    })
  }

  handleNewRecord = () => {
    this.setState({ user: {} })
  }

  handleUndoRecord = () => {
    this.setState({user: this.state.undo})
  }

  handleDeleteRecord = () => {
    let {user} = this.state
    this.setState(
      { spinMessage: "Deleting User..." },
      () => {
        this.setState({
          user: {},
          users: this.state.users.filter((x) => x.id !== user.id),
          spinMessage: null
        })
    })
  }


  handleSaveRecord = (record, onSuccess) => {
    let {users, user} = this.state
    if (record.id) {
      // Update
      this.setState(
        { spinMessage: "Updating User..." },
        () => {
        this.setState({
          user: record,
          undo: user,
          users: users.map((x) => (x.id === record.id ? user : x)),
          spinMessage: null,
          })
          onSuccess()
      })
    } 
    else {
      // Insert
      this.setState(
        { spinMessage: "Creating User..." }, 
        () => {
        this.setState({
          user: record,
          undo: user,
          users: [users, record],
          spinMessage: null,
          })
          onSuccess()
      })
    }
  }

  render() {
    return (
      <Layout.Content>
        <Spin
          tip={this.state.spinMessage}
          spinning={this.state.spinMessage ? true : false}
        >
          <Container
            leftSpan="10"
            sider={() => (
              <React.Fragment>
                <RecordSelector
                  dataSource={this.state.users}
                  columns={this.columns}
                  onSelect={this.handleSelectRecord}
                  message={this.state.spinMessage}
                  form={this.props.form}
                />
              </React.Fragment>
            )}
          >
          <UserForm
            user={this.state.user}
            users={this.state.users}
            genders={this.state.genders}
            onNew={this.handleNewRecord}
            onUndo={this.handleUndoRecord}
            onDelete={this.handleDeleteRecord}
            onSave={this.handleSaveRecord}
            message={this.state.spinMessage}
            form={this.props.form}
            />
          </Container>
        </Spin>
      </Layout.Content>
    )
  }

}

User = Form.create()(User)
export default User


const NullAsEmpty = (text) => {
  return text ? text : ''
}

// convert null to 0 
const NullAsZero = (val) => {
  return val ? val : 0
}