import React, { Component } from "react"
import { Form, Input, Select, Switch } from "antd"
import DataEntryForm from "wp-data-entry-form"
import { defaultFormItemLayout } from "./constant"

const FormItem = Form.Item
const Option = Select.Option

class UserForm extends Component {
  render() {
    const { getFieldDecorator } = this.props.form

    return (
      <DataEntryForm
        id="user"
        message={this.props.message}
        form={this.props.form}
        onNew={this.props.onNew}
        onUndo={this.props.onUndo}
        onDelete={this.props.onDelete}
        onSave={this.props.onSave}
      >
        <br></br>
        <FormItem 
          {...defaultFormItemLayout} 
          label="Id "
        >
          {getFieldDecorator("id", {
            initialValue: this.props.user.id,
            value: this.props.user.id,
          })(
            <Input 
              placeholder="Id" 
              disabled={true} 
            />
          )}
        </FormItem>
        <FormItem 
          {...defaultFormItemLayout} 
          label="Name "
        >
          {getFieldDecorator("name", {
            initialValue: this.props.user.name,
            value: this.props.user.name,
            rules: [{ required: true, message: "Please enter a name" }],
          })(
            <Input placeholder="Name" />
          )}
        </FormItem>
        <FormItem 
          {...defaultFormItemLayout} 
          label="Username ">
          {getFieldDecorator("username", {
            initialValue: this.props.user.username,
            value: this.props.user.username,
            rules: [{ required: true, message: "Please enter a username" }],
          })(
            <Input placeholder="Username" />
          )}
        </FormItem>
        <FormItem 
          {...defaultFormItemLayout} 
          label="Password ">
          {getFieldDecorator("password", {
            initialValue: this.props.user.password,
            value: this.props.user.password,
            rules: [{ required: true, message: "Please enter a password" }],
          })(
            <Input placeholder="Password" />
          )}
        </FormItem>
        <FormItem 
          {...defaultFormItemLayout} 
          label="Gender ">
          {getFieldDecorator("gender", {
            initialValue: this.props.user.gender,
            value: this.props.user.gender,
            rules: [{ required: true, message: "Please select a gender" }],
          })(
            <Select 
              placeholder='Select Gender'
              style={{width: '100%'}}
              showSearch
              optionFilterProp="children"
            > 
              {
                this.props.genders.length > 0
                  ? this.props.genders.map(x => <Option key={x.id} value={x.id}>{x.name}</Option>)
                  : ""
              }
            </Select>
          )}
        </FormItem>
        <FormItem 
          {...defaultFormItemLayout} 
          label="Active ">
          {getFieldDecorator("active", {
            initialValue: this.props.user.active,
            value: this.props.user.active,
            valuePropName: 'checked' 
          })(
            <Switch />
          )}
        </FormItem>
      </DataEntryForm>
    )
  }
}

export default UserForm
